"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

require("core-js/stable");

require("regenerator-runtime/runtime");

var _express = _interopRequireWildcard(require("express"));

var _helmet = _interopRequireDefault(require("helmet"));

var _cors2 = _interopRequireDefault(require("cors"));

var _compression = _interopRequireDefault(require("compression"));

var _doBootstrapHandler = _interopRequireDefault(require("./Handlers/doBootstrapHandler"));

var _doNotFoundHandler = _interopRequireDefault(require("./Handlers/doNotFoundHandler"));

var _doStrapHandler = _interopRequireDefault(require("./Handlers/doStrapHandler"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const createExpressMicroUI = ({
  config,
  profile = 'local',
  logger = console
}) => {
  var _env$api, _env$api2, _env$api3;

  // Retrieve the environment profiles
  const env = config.environments.profiles[profile] || config.environments.profiles[config.environments.default]; // Allow for env overrides

  const _cors = ((_env$api = env.api) === null || _env$api === void 0 ? void 0 : _env$api.cors) || config.api.cors;

  const _port = ((_env$api2 = env.api) === null || _env$api2 === void 0 ? void 0 : _env$api2.port) || config.api.port;

  const _messages = ((_env$api3 = env.api) === null || _env$api3 === void 0 ? void 0 : _env$api3.messages) || config.api.messages; // Attempt to start the express server


  try {
    var _env$api4;

    // Retrieve any api env
    const apiEnv = ((_env$api4 = env.api) === null || _env$api4 === void 0 ? void 0 : _env$api4.env) || {}; // Update the process env with any provided api env

    process.env = { ...process.env,
      ...apiEnv
    }; // Saying hello

    logger.info(_messages.START_UP); // Create a new express instance

    const api = (0, _express.default)(); // Setting up middlewares

    api.use((0, _express.json)());
    api.use((0, _helmet.default)());
    api.use((0, _cors2.default)(_cors));
    api.use((0, _compression.default)()); // Serve static assets

    api.use(_express.default.static('./.microui')); // Hydrate and output the bootstrapper script

    api.get('/bootstrap.js', (0, _doBootstrapHandler.default)(env, config)); // Adds a route to a router of sorts

    const route = (path, method, handler) => {
      // Sanity check to ensure only valid methods are passed
      if (!['get', 'post', 'all'].includes(method.toLowerCase())) {
        throw new Error('METHOD_NOT_SUPPORTED');
      } // Wrap the handler in
      // @ts-ignore


      api[method.toLowerCase()](path, (req, res) => handler({
        platform: 'express',
        req,
        res,
        logger,
        env,
        config,
        method
      }));
    }; // Straps a component into the SSR api


    const strap = (name, component) => {
      // Handle a GET request to fetch a component
      api.get(`/${name}`, (0, _doStrapHandler.default)(name, component, logger, env, config, 'GET')); // Handle a POST request to fetch a component

      api.post(`/${name}`, (0, _doStrapHandler.default)(name, component, logger, env, config, 'POST'));
    }; // Boots up the server


    const boot = () => {
      // Handle any 404 errors
      api.use(_doNotFoundHandler.default); // Start the server listening on the provided port

      api.listen(_port); // Log that something happened

      logger.info(_messages.STARTED_UP, _port);
    }; // Returns the instance of the server, the strapper the booter, the config and the logger


    return {
      api,
      strap,
      boot,
      route,
      env,
      config: config,
      logger
    }; // If the application throws an error
    // We catch and log for debugging
  } catch (e) {
    // Log out the thrown error
    logger.error(_messages.CRASHED, e.message);
  }
};

var _default = createExpressMicroUI;
exports.default = _default;